@@ -7,4 +7,8 @@ from pytconf.config import (
     config_arg_parse_and_launch,
     register_endpoint,
     get_free_args,
+    write_config_file_json_user,
+    write_config_file_json_system,
+    rm_config_file_json_user,
+    rm_config_file_json_system,
 )
