@@ -17,15 +17,27 @@ from pytconf.color_utils import (
 )
 from pytconf.errors_collector import ErrorsCollector
 from pytconf.param import Param, NO_DEFAULT
+from pytconf.pydoc import get_first_line
 from pytconf.utils import get_logger
 
 PARAMS_ATTRIBUTE = "_params"
-SPECIAL_COMMANDS = {"help", "help-suggest", "help-all"}
+
+DEFAULT_FUNCTION_GROUP_NAME = "default"
+DEFAULT_FUNCTION_GROUP_DESCRIPTION = "default command group"
+DEFAULT_FUNCTION_GROUP_SHOW_META = False
+DEFAULT_FUNCTION_GROUP_SHOW = True
+
+SPECIAL_FUNCTION_GROUP_NAME = "special"
+SPECIAL_FUNCTION_GROUP_DESCRIPTION = "special command group"
+SPECIAL_FUNCTION_GROUP_SHOW_META = False
+SPECIAL_FUNCTION_GROUP_SHOW = False
+
+DEFAULT_FUNCTION_DESCRIPTION = "No function description available"
 
 
 class MetaConfig(type):
     """
-    base class for all configs
+    Meta class for all configs
     """
 
     def __new__(mcs, name, bases, cls_dict):
@@ -53,14 +65,12 @@ class MetaConfig(type):
 
 class Config(metaclass=MetaConfig):
     """
-        base class for all configs
+    Base class for all configs
     """
 
     @classmethod
     def get_attributes(cls: Any) -> List[str]:
         return getattr(cls, PARAMS_ATTRIBUTE).keys()
-        # return [attr for attr in dir(cls) if not callable(getattr(cls, attr))
-        #        and not attr.startswith("__")]
 
     @classmethod
     def get_params(cls: Any) -> Dict[str, Param]:
@@ -80,43 +90,63 @@ class HtmlGen:
         self.line = line
 
 
-def get_first_line(doc: Union[str, None]) -> Union[str, None]:
-    if doc is None:
-        return None
-    lines = doc.split("\n")
-    for line in lines:
-        if line == "" or line.isspace():
-            continue
-        return line.strip()
-    return None
-
-
 class PytconfConf:
     def __init__(self):
         self._configs = set()
         self._config_names = set()
-        self.main_function = None
-        self.main_description = None
+        self.main_function: Union[Callable, None] = None
+        self.main_description: str = "No application description"
         self.function_name_to_configs: Dict[str, List[Type[Config]]] = dict()
         self.function_name_to_suggest_configs: Dict[str, List[Type[Config]]] = dict()
         self.function_name_to_callable: Dict[str, Callable] = dict()
-        self.function_group_names: Dict[Union[str, None], Set[str]] = defaultdict(set)
         self.allow_free_args: Dict[str, bool] = dict()
         self.min_free_args: Dict[str, Union[int, None]] = dict()
         self.max_free_args: Dict[str, Union[int, None]] = dict()
-        self.description: Dict[str, Union[str, None]] = dict()
+        self.description: Dict[str, str] = dict()
+
+        self.function_group_names: Dict[str, Set[str]] = defaultdict(set)
+        self.function_group_list: List[str] = []
         self.function_group_descriptions: Dict[str, str] = dict()
-        self.function_group_list: List[Union[str, None]] = [None]
+        self.function_group_show_meta: Dict[str, bool] = dict()
+        self.function_group_show: Dict[str, bool] = dict()
+
         self.attribute_to_config: Dict[str, Type[Config]] = dict()
         self.free_args: List[str] = []
-        self.app_name: Union[str, None] = None
+        self.app_name: str = "No application name"
+        self.version: str = "No version"
 
-    def register_main(self,
-                      main_function: Callable,
-                      main_description: Union[str, None] = None,
-                      ):
+    def register_main(
+        self,
+        main_function: Callable,
+        main_description: str,
+        app_name: str,
+        version: str,
+    ):
         self.main_function = main_function
         self.main_description = main_description
+        self.app_name = app_name
+        self.version = version
+
+    def register_function(
+        self,
+        function: Callable,
+        description: str,
+        name: str,
+        configs: List[Type[Config]] = (),
+        suggest_configs: List[Type[Config]] = (),
+        group: Union[str, None] = None,
+        allow_free_args: bool = False,
+        min_free_args: Union[int, None] = None,
+        max_free_args: Union[int, None] = None,
+    ):
+        self.description[name] = description
+        self.function_name_to_callable[name] = function
+        self.function_name_to_configs[name] = configs
+        self.function_name_to_suggest_configs[name] = suggest_configs
+        self.function_group_names[group].add(name)
+        self.allow_free_args[name] = allow_free_args
+        self.min_free_args[name] = min_free_args
+        self.max_free_args[name] = max_free_args
 
     def register_config(self, config: Type[Config], name):
         """
@@ -140,55 +170,32 @@ class PytconfConf:
         for error in errors.errors:
             print_error(error)
 
-    def get_main_description(self):
-        if self.main_description is not None:
-            return self.main_description
-        doc = get_first_line(self.main_function.__doc__)
-        if doc is not None:
-            return doc
-        # doc = "\n".join(map(lambda x: f"  {x.strip()}", doc.split("\n")))
-        return None
-
     def show_help(self) -> None:
-        print(f"Usage: {self.app_name} [OPTIONS] COMMAND [ARGS]...")
-        main_doc = self.get_main_description()
-        if main_doc is not None:
-            print_highlight(f"\n  {main_doc}")
-        print("\nOptions:")
-        print("  --help         Show mandatory help")
-        print("  --help-suggest Show mandatory+suggestions help")
-        print("  --help-all     Show all help")
-        print("\nCommands:")
-        single_group = len(self.function_group_list) == 1
+        print(f"Usage: {color_hi(self.app_name)} COMMAND [ARGS]...")
+        space = " " * 2
+        print_highlight(f"\n{space}{self.main_description}\n")
         for function_group in self.function_group_list:
-            if not single_group:
+            show = self.function_group_show[function_group]
+            if not show:
+                continue
+            show_meta = self.function_group_show_meta[function_group]
+            if show_meta:
                 description = self.function_group_descriptions[function_group]
-                print(f"  {function_group}: {description}")
+                print(f"{space}{function_group}: {description}")
+                cmd_space = space + " " * 2
+            else:
+                cmd_space = space
             for name in sorted(self.function_group_names[function_group]):
-                function_doc = self.get_function_description(name)
-                if function_doc is None:
-                    print_highlight(f"  {name}")
-                else:
-                    print(f"  {color_hi(name)}: {function_doc}")
+                function_doc = self.description[name]
+                print(f"{cmd_space}{color_hi(name)}: {function_doc}")
             print()
 
-    def get_function_description(self, name: str):
-        description = self.description[name]
-        if description is not None:
-            return description
-        f = self.function_name_to_callable[name]
-        doc = get_first_line(f.__doc__)
-        if doc is not None:
-            return doc
-        return None
-
     def show_help_for_function(
         self, function_name: str, show_help_full: bool, show_help_suggest: bool
     ) -> None:
         print(f"Usage: {self.app_name} {function_name} [OPTIONS] [ARGS]...")
-        function_doc = self.get_function_description(function_name)
-        if function_doc is not None:
-            print_highlight(f"\n  {function_doc}")
+        function_doc = self.description[function_name]
+        print_highlight(f"\n  {function_doc}")
         print("\nOptions:\n")
         for config in self.function_name_to_configs[function_name]:
             self.show_help_for_config(config)
@@ -278,32 +285,48 @@ class PytconfConf:
 
     def get_system_config(self):
         return f"/etc/{self.app_name}.json"
-    
+
     def get_user_config(self):
         return os.path.expanduser(f"~/.config/{self.app_name}.json")
 
+    def register_function_group(
+        self,
+        name: str,
+        description: str,
+        show_meta: bool,
+        show: bool,
+    ):
+        self.function_group_list.append(name)
+        self.function_group_descriptions[name] = description
+        self.function_group_show_meta[name] = show_meta
+        self.function_group_show[name] = show
+
+    def register_defaults(self):
+        self.register_function_group(
+            name=DEFAULT_FUNCTION_GROUP_NAME,
+            description=DEFAULT_FUNCTION_GROUP_DESCRIPTION,
+            show_meta=DEFAULT_FUNCTION_GROUP_SHOW_META,
+            show=DEFAULT_FUNCTION_GROUP_SHOW,
+        )
+        self.register_function_group(
+            name=SPECIAL_FUNCTION_GROUP_NAME,
+            description=SPECIAL_FUNCTION_GROUP_DESCRIPTION,
+            show_meta=SPECIAL_FUNCTION_GROUP_SHOW_META,
+            show=SPECIAL_FUNCTION_GROUP_SHOW,
+        )
+
     def config_arg_parse_and_launch(
-        self, args: Union[List[str], None] = None, launch=True, app_name=None,
+        self,
+        args: Union[List[str], None] = None,
+        launch=True,
+        do_exit=True,
     ) -> None:
-        # if we are given no args then take the from sys.argv
+
+        self.register_defaults()
         if args is None:
-            args = sys.argv
-
-        # set app name
-        if app_name is None:
-            if args:
-                self.app_name = os.path.basename(args[0])
-            else:
-                self.app_name = "UNKNOWN APP NAME"
-        else:
-            self.app_name = app_name
-
-        # we don't need the first argument which is the script path
-        if args:
-            args = args[1:]
+            args = sys.argv[1:]
 
         flags: Dict[str, str] = dict()
-        special_flags = set()
         errors = ErrorsCollector()
         self.free_args = []
 
@@ -311,11 +334,8 @@ class PytconfConf:
         self.read_flags_from_config(file_name=self.get_system_config(), flags=flags)
         self.read_flags_from_config(file_name=self.get_user_config(), flags=flags)
 
-        # get the command (it's the first argument if it doesnt start with --
-        # if there are free args then the command is the first of these
-        # find the selected command
         command_selected = None
-        if not args[0].startswith("--"):
+        if len(args) > 0:
             command = args.pop(0)
             if command in self.function_name_to_callable:
                 command_selected = command
@@ -323,25 +343,7 @@ class PytconfConf:
                 errors.add_error(f"unknown command [{command}]")
 
         # now parse the args
-        self.parse_args(args, errors, flags, special_flags)
-
-        # handle help flags
-        show_help = False
-        show_help_full = False
-        show_help_suggest = False
-
-        if "help" in special_flags:
-            show_help = True
-        if "help-suggest" in special_flags:
-            show_help = True
-            show_help_suggest = True
-        if "help-all" in special_flags:
-            show_help = True
-            show_help_full = True
-
-        # if there are no free args and just one function then this is it
-        if len(self.function_name_to_callable) == 1:
-            command_selected = list(self.function_name_to_callable.keys())[0]
+        self.parse_args(args, errors, flags)
 
         # if we have command we can check free args errors
         if command_selected is not None:
@@ -358,30 +360,26 @@ class PytconfConf:
                 if len(self.free_args) > 0:
                     errors.add_error(f"free args are not allowed [{self.free_args}]")
 
+        do_help = False
         if command_selected is None:
-            if not show_help:
-                errors.add_error("no command is selected")
+            errors.add_error("no command is selected")
+            do_help = True
         else:
             self.process_flags(command_selected, flags, errors)
 
-        if errors.have_errors() and not show_help:
+        if errors.have_errors():
             self.print_errors(errors)
-            return
-
-        if show_help:
-            if command_selected:
-                self.show_help_for_function(
-                    command_selected, show_help_full, show_help_suggest,
-                )
-            else:
+            if do_help:
                 self.show_help()
+            if do_exit:
+                sys.exit(1)
             return
 
         if launch:
             function_to_run = self.function_name_to_callable[command_selected]
             function_to_run()
 
-    def parse_args(self, args, errors, flags, special_flags):
+    def parse_args(self, args, errors, flags):
         free_args_started = False
         while args:
             current = args.pop(0)
@@ -395,8 +393,6 @@ class PytconfConf:
                     if args:
                         more = args.pop(0)
                         flags[real] = more
-                    elif real in SPECIAL_COMMANDS:
-                        special_flags.add(real)
                     else:
                         errors.add_error(
                             f"argument [{real}] needs a follow-up argument"
@@ -409,9 +405,8 @@ class PytconfConf:
 
     def get_html(self) -> str:
         html_gen = HtmlGen()
-        main_doc = self.get_main_description()
-        if main_doc is not None:
-            html_gen.line("h1", main_doc)
+        main_doc = self.main_description
+        html_gen.line("h1", main_doc)
         html_gen.line("h2", "API specifications")
         single_group = len(self.function_group_list) == 1
         with html_gen.tag("ul"):
@@ -472,7 +467,7 @@ class PytconfConf:
                     html_gen.line("td", default)
                     html_gen.line("td", more_help)
 
-    def write_config_file_json(self, filename):
+    def write_config_file_json(self, filename: str) -> None:
         values: Dict[str, str] = dict()
         for config in self._configs:
             for name, param in config.get_params().items():
@@ -481,10 +476,10 @@ class PytconfConf:
         with open(filename, "wt") as f:
             json.dump(values, f, indent=4)
 
-    def write_config_file_json_user(self):
+    def write_config_file_json_user(self) -> None:
         self.write_config_file_json(self.get_user_config())
 
-    def write_config_file_json_system(self):
+    def write_config_file_json_system(self) -> None:
         self.write_config_file_json(self.get_system_config())
 
 
@@ -495,12 +490,12 @@ def get_pytconf():
     return _pytconf
 
 
-def config_arg_parse_and_launch(launch=True, args=None, app_name=None) -> None:
+def config_arg_parse_and_launch(launch=True, args=None) -> None:
     """
     This is the real API
     """
     get_pytconf().config_arg_parse_and_launch(
-        launch=launch, args=args, app_name=app_name,
+        launch=launch, args=args,
     )
 
 
@@ -508,93 +503,117 @@ def get_free_args() -> List[str]:
     return get_pytconf().free_args
 
 
+# TODO: remove default values for next function when ready to force
+# TODO: change function_group_name -> name
+# TODO: change function_group_description -> description
 def register_function_group(
-    function_group_name: str, function_group_description: str
+    function_group_name: str,
+    function_group_description: str,
+    show_meta: bool = True,
+    show: bool = True,
 ) -> None:
-    pt = get_pytconf()
-    pt.function_group_descriptions[function_group_name] = function_group_description
-    pt.function_group_list.append(function_group_name)
+    get_pytconf().register_function_group(
+        name=function_group_name,
+        description=function_group_description,
+        show_meta=show_meta,
+        show=show,
+    )
 
 
-def register_main(main_description: Union[str, None] = None) -> Callable[[Any], Any]:
+def register_main(
+    main_description: str = "NO DESCRIPTION",  # TODO: remove when ready to force
+    app_name: str = "NO APP",  # TODO: remove when ready to force
+    version: str = "NO VERSION",  # TODO: remove when ready to force
+) -> Callable[[Any], Any]:
     def identity(main_function):
         get_pytconf().register_main(
             main_function=main_function,
             main_description=main_description,
+            app_name=app_name,
+            version=version,
         )
         return main_function
+
     return identity
 
 
 def register_endpoint(
+    description: str = "No description",  # TODO: remove default value
+    name: Union[str, None] = None,
     configs: List[Type[Config]] = (),
     suggest_configs: List[Type[Config]] = (),
-    group: Union[str, None] = None,
+    group: str = DEFAULT_FUNCTION_GROUP_NAME,
     allow_free_args: bool = False,
     min_free_args: Union[int, None] = None,
     max_free_args: Union[int, None] = None,
-    description: Union[str, None] = None,
 ) -> Callable[[Any], Any]:
     logger = get_logger()
     logger.debug("registering endpoint")
 
-    def identity(f):
+    def identity(function):
+        if name is None:
+            function_name = function.__name__
+        else:
+            function_name = name
         register_function(
-            f,
+            function=function,
+            description=description,
+            name=function_name,
             configs=configs,
             suggest_configs=suggest_configs,
             group=group,
             allow_free_args=allow_free_args,
             min_free_args=min_free_args,
             max_free_args=max_free_args,
-            description=description,
         )
-        return f
+        return function
 
     return identity
 
 
 def register_function(
-    f: Callable,
+    function: Callable,
+    description: str,
+    name: str,
     configs: List[Type[Config]] = (),
     suggest_configs: List[Type[Config]] = (),
     group: Union[str, None] = None,
     allow_free_args: bool = False,
     min_free_args: Union[int, None] = None,
     max_free_args: Union[int, None] = None,
-    description: Union[str, None] = None,
 ) -> None:
-    function_name = f.__name__
-    pt = get_pytconf()
-    pt.function_name_to_callable[function_name] = f
-    pt.function_name_to_configs[function_name] = configs
-    pt.function_name_to_suggest_configs[function_name] = suggest_configs
-    pt.function_group_names[group].add(function_name)
-    pt.allow_free_args[function_name] = allow_free_args
-    pt.min_free_args[function_name] = min_free_args
-    pt.max_free_args[function_name] = max_free_args
-    pt.description[function_name] = description
+    get_pytconf().register_function(
+        function=function,
+        description=description,
+        name=name,
+        configs=configs,
+        suggest_configs=suggest_configs,
+        group=group,
+        allow_free_args=allow_free_args,
+        min_free_args=min_free_args,
+        max_free_args=max_free_args,
+    )
 
 
-def write_config_file_json_user():
+def write_config_file_json_user() -> None:
     filename = get_pytconf().get_user_config()
     if not os.path.isfile(filename):
         get_pytconf().write_config_file_json_user()
 
 
-def rm_config_file_json_user():
+def rm_config_file_json_user() -> None:
     filename = get_pytconf().get_user_config()
     if os.path.isfile(filename):
         os.unlink(filename)
 
 
-def write_config_file_json_system():
+def write_config_file_json_system() -> None:
     filename = get_pytconf().get_system_config()
     if not os.path.isfile(filename):
         get_pytconf().write_config_file_json_system()
 
 
-def rm_config_file_json_system():
+def rm_config_file_json_system() -> None:
     filename = get_pytconf().get_system_config()
     if os.path.isfile(filename):
         os.unlink(filename)
